@@ -12,13 +12,17 @@ from typing import Dict, Tuple, List, Optional, Type
 from setproctitle import setproctitle as ptitle
 
 from allenact import __version__
-from allenact.algorithms.onpolicy_sync.runner import OnPolicyRunner, _CONFIG_KWARGS_STR
+from allenact.algorithms.onpolicy_sync.runner import (
+    OnPolicyRunner,
+    CONFIG_KWARGS_STR,
+    SaveDirFormat,
+)
 from allenact.base_abstractions.experiment_config import ExperimentConfig
 from allenact.utils.system import get_logger, init_logging, HUMAN_LOG_LEVELS
 
 
-def get_args():
-    """Creates the argument parser and parses any input arguments."""
+def get_argument_parser():
+    """Creates the argument parser."""
 
     # noinspection PyTypeChecker
     parser = argparse.ArgumentParser(
@@ -51,7 +55,7 @@ def get_args():
         help="sometimes it is useful to be able to pass additional key-word arguments"
         " to `__init__` when initializing an experiment configuration. This flag can be used"
         " to pass such key-word arugments by specifying them with json, e.g."
-        "\n\t--config_kwargs \"{'gpu_id': 0, 'my_important_variable': [1,2,3]}\""
+        '\n\t--config_kwargs \'{"gpu_id": 0, "my_important_variable": [1,2,3]}\''
         "\nTo see which arguments are supported for your experiment see the experiment"
         " config's `__init__` function. If the value passed to this function is a file path"
         " then we will try to load this file path as a json object and use this json object"
@@ -64,8 +68,8 @@ def get_args():
         default="",
         required=False,
         help="Add an extra tag to the experiment when trying out new ideas (will be used"
-        "as a subdirectory of the tensorboard path so you will be able to"
-        "search tensorboard logs using this extra tag). This can also be used to add an extra"
+        " as a subdirectory of the tensorboard path so you will be able to"
+        " search tensorboard logs using this extra tag). This can also be used to add an extra"
         " organization when running evaluation (e.g. `--extra_tag running_eval_on_great_idea_12`)",
     )
 
@@ -78,6 +82,16 @@ def get_args():
         help="experiment output folder",
     )
 
+    parser.add_argument(
+        "--save_dir_fmt",
+        required=False,
+        type=lambda s: SaveDirFormat[s.upper()],
+        default="flat",
+        help="The file structure to use when saving results from allenact."
+        " See documentation o f`SaveDirFormat` for more details."
+        " Allowed values are ('flat' and 'nested'). Default: 'flat'.",
+    )
+
     parser.add_argument(
         "-s", "--seed", required=False, default=None, type=int, help="random seed",
     )
@@ -101,11 +115,18 @@ def get_args():
         "\n(2) the path to a directory of checkpoint files all of which you'd like to be evaluated"
         " (checkpoints are expected to have a `.pt` file extension),"
         '\n(3) a "glob" pattern (https://tldp.org/LDP/abs/html/globbingref.html) that will be expanded'
-        "using python's `glob.glob` function and should return a collection of checkpoint files."
+        " using python's `glob.glob` function and should return a collection of checkpoint files."
         "\nIf you'd like to only evaluate a subset of the checkpoints specified by the above directory/glob"
         " (e.g. every checkpoint saved after 5mil steps) you'll likely want to use the `--approx_ckpt_step_interval`"
         " flag.",
     )
+    parser.add_argument(
+        "--infer_output_dir",
+        dest="infer_output_dir",
+        action="store_true",
+        required=False,
+        help="applied when evaluating checkpoint(s) in nested save_dir_fmt: if specified, the output dir will be inferred from checkpoint path.",
+    )
     parser.add_argument(
         "--approx_ckpt_step_interval",
         required=False,
@@ -125,8 +146,8 @@ def get_args():
         action="store_true",
         required=False,
         help="for training, if checkpoint is specified, DO NOT continue the training pipeline from where"
-        "training had previously ended. Instead restart the training pipeline from scratch but"
-        "with the model weights from the checkpoint.",
+        " training had previously ended. Instead restart the training pipeline from scratch but"
+        " with the model weights from the checkpoint.",
     )
     parser.set_defaults(restart_pipeline=False)
 
@@ -190,10 +211,49 @@ def get_args():
     )
     parser.set_defaults(disable_config_saving=False)
 
+    parser.add_argument(
+        "--collect_valid_results",
+        dest="collect_valid_results",
+        action="store_true",
+        required=False,
+        help="enables returning and saving valid results during training",
+    )
+    parser.set_defaults(collect_valid_results=False)
+
+    parser.add_argument(
+        "--test_expert",
+        dest="test_expert",
+        action="store_true",
+        required=False,
+        help="use expert during test",
+    )
+    parser.set_defaults(test_expert=False)
+
     parser.add_argument(
         "--version", action="version", version=f"allenact {__version__}"
     )
 
+    parser.add_argument(
+        "--distributed_ip_and_port",
+        dest="distributed_ip_and_port",
+        required=False,
+        type=str,
+        default="127.0.0.1:0",
+        help="IP address and port of listener for distributed process with rank 0."
+        " Port number 0 lets runner choose a free port. For more details, please follow the"
+        " tutorial https://allenact.org/tutorials/distributed-objectnav-tutorial/.",
+    )
+
+    parser.add_argument(
+        "--machine_id",
+        dest="machine_id",
+        required=False,
+        type=int,
+        default=0,
+        help="ID for machine in distributed runs. For more details, please follow the"
+        " tutorial https://allenact.org/tutorials/distributed-objectnav-tutorial/",
+    )
+
     ### DEPRECATED FLAGS
     parser.add_argument(
         "-t",
@@ -221,6 +281,13 @@ def get_args():
     )
     ### END DEPRECATED FLAGS
 
+    return parser
+
+
+def get_args():
+    """Creates the argument parser and parses any input arguments."""
+
+    parser = get_argument_parser()
     args = parser.parse_args()
 
     # check for deprecated
@@ -301,10 +368,11 @@ def load_config(args) -> Tuple[ExperimentConfig, Dict[str, str]]:
             sm for sm in all_sub_modules if desired_config_name in os.path.basename(sm)
         ]
         raise ModuleNotFoundError(
-            "Could not import experiment '{}', are you sure this is the right path?"
-            " Possibly relevant files include {}.".format(
-                module_path, relevant_submodules
-            ),
+            f"Could not import experiment '{module_path}', are you sure this is the right path?"
+            f" Possibly relevant files include {relevant_submodules}."
+            f" Note that the experiment must be reachable along your `PYTHONPATH`, it might"
+            f" be helpful for you to run `export PYTHONPATH=$PYTHONPATH:$PWD` in your"
+            f" project's top level directory."
         ) from e
 
     experiments = [
@@ -328,7 +396,7 @@ def load_config(args) -> Tuple[ExperimentConfig, Dict[str, str]]:
                 get_logger().warning(
                     f"The input for --config_kwargs ('{args.config_kwargs}')"
                     f" does not appear to be valid json. Often this is due to"
-                    f" json requiring very specific synax (e.g. double quoted strings)"
+                    f" json requiring very specific syntax (e.g. double quoted strings)"
                     f" we'll try to get around this by evaluating with `ast.literal_eval`"
                     f" (a safer version of the standard `eval` function)."
                 )
@@ -340,7 +408,7 @@ def load_config(args) -> Tuple[ExperimentConfig, Dict[str, str]]:
 
     config = experiments[0](**config_kwargs)
     sources = _config_source(config_type=experiments[0])
-    sources[_CONFIG_KWARGS_STR] = json.dumps(config_kwargs)
+    sources[CONFIG_KWARGS_STR] = json.dumps(config_kwargs)
     return config, sources
 
 
@@ -359,6 +427,7 @@ def main():
         OnPolicyRunner(
             config=cfg,
             output_dir=args.output_dir,
+            save_dir_fmt=args.save_dir_fmt,
             loaded_config_src_files=srcs,
             seed=args.seed,
             mode="train",
@@ -367,15 +436,19 @@ def main():
             extra_tag=args.extra_tag,
             disable_tensorboard=args.disable_tensorboard,
             disable_config_saving=args.disable_config_saving,
+            distributed_ip_and_port=args.distributed_ip_and_port,
+            machine_id=args.machine_id,
         ).start_train(
             checkpoint=args.checkpoint,
             restart_pipeline=args.restart_pipeline,
             max_sampler_processes_per_worker=args.max_sampler_processes_per_worker,
+            collect_valid_results=args.collect_valid_results,
         )
     else:
         OnPolicyRunner(
             config=cfg,
             output_dir=args.output_dir,
+            save_dir_fmt=args.save_dir_fmt,
             loaded_config_src_files=srcs,
             seed=args.seed,
             mode="test",
@@ -384,10 +457,14 @@ def main():
             extra_tag=args.extra_tag,
             disable_tensorboard=args.disable_tensorboard,
             disable_config_saving=args.disable_config_saving,
+            distributed_ip_and_port=args.distributed_ip_and_port,
+            machine_id=args.machine_id,
         ).start_test(
             checkpoint_path_dir_or_pattern=args.checkpoint,
+            infer_output_dir=args.infer_output_dir,
             approx_ckpt_step_interval=args.approx_ckpt_step_interval,
             max_sampler_processes_per_worker=args.max_sampler_processes_per_worker,
+            inference_expert=args.test_expert,
         )
 
 
